"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function defaultInitial(instance) {
  const className = instance.props.className;
  return _react.default.createElement('span', {
    className: `${className ? `${className} ` : ''}react-vue-like-async-placeholder`
  });
}

class ReactVueLikeAsync extends _react.default.Component {
  constructor(props) {
    super(props);
    this.mounted = false;
    this.counter = 0;
    this.state = {
      error: null,
      result: props.initial || null,
      pending: null,
      isResolved: false
    };
  }

  componentDidMount() {
    this.mounted = true;
    if (this.props.promise) this.start();
  }

  componentDidUpdate(prevProps) {
    var _this = this;

    const promise = this.props.promise;

    if (promise !== prevProps.promise) {
      return promise ? this.start() : null;
    }

    ['initial', 'error', 'loading'].some(function (key) {
      let current = _this.props[key];

      if (prevProps[key] !== current && _this.state.result === current) {
        _this.setState({
          result: current
        });

        return true;
      }
    });
  }

  start() {
    var _this2 = this;

    let _this$props = this.props,
        promise = _this$props.promise,
        initial = _this$props.initial,
        loading = _this$props.loading,
        error = _this$props.error;
    const counter = ++this.counter;

    const isCanceled = function () {
      return _this2.state.pending !== promise || !_this2.mounted || counter !== _this2.counter;
    };

    const asyncs = promise && promise.asyncs;

    if (asyncs) {
      if (asyncs.initial !== undefined) initial = asyncs.initial;
      if (asyncs.loading !== undefined) loading = asyncs.loading;
      if (asyncs.error !== undefined) error = asyncs.error;
    }

    if (promise instanceof Promise) {
      this.setState({
        isResolved: false,
        pending: promise,
        result: loading || initial || null,
        error: null
      });

      if (promise) {
        ((0, _utils.isFunction)(promise) ? promise : function () {
          return promise;
        })(this).then(function (result) {
          if (isCanceled()) return;

          _this2.setState({
            isResolved: true,
            result,
            pending: null
          });
        }).catch(function (ex) {
          if (isCanceled()) return;

          if (process.env.NODE_ENV !== 'production') {
            (0, _utils.warn)('[async] promise throw error:', ex);
          }

          _this2.setState({
            isResolved: true,
            error: ex,
            result: error || null,
            pending: null
          });
        });
      }
    } else this.setState({
      isResolved: true,
      result: promise,
      pending: null
    });
  }

  componentWillUnmount() {
    this.mounted = false;
  }

  render() {
    let result = this.state.result;
    if (result && result.__esModule) result = result.default;
    if ((0, _utils.isFunction)(result)) result = result(this);
    return result === undefined ? null : result;
  }

}

_defineProperty(ReactVueLikeAsync, "propTypes", {
  className: _propTypes.default.any,
  promise: _propTypes.default.oneOfType([_propTypes.default.func, _propTypes.default.instanceOf(Promise)]).isRequired,
  initial: _propTypes.default.any,
  error: _propTypes.default.any,
  loading: _propTypes.default.any
});

_defineProperty(ReactVueLikeAsync, "defaultProps", {
  initial: defaultInitial
});

(0, _utils.innumerable)(ReactVueLikeAsync, '__vuelikeAsyncClass', true);
var _default = ReactVueLikeAsync;
exports.default = _default;