"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/web.dom.iterable");

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _mobxReact = require("mobx-react");

var _mobx = require("./mobx");

var _utils = require("./utils");

var _config = _interopRequireDefault(require("./config"));

var _class, _class2, _temp;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(n); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function checkUnmount(fn) {
  if (!fn) return fn;
  let old;
  return function get() {
    if (this && this.$isWillUnmount) return old;
    return old = fn.apply(this, arguments);
  };
}

function generateComputed(obj, propData, data, methods, target) {
  var _this = this;

  const ret = {};
  Object.keys(obj).forEach(function (key) {
    if (RESERVED_KEYS.includes(key)) {
      let e = new Error(`key '${key}' in computed is reserved keywords`);
      (0, _utils.handleError)(e, _this, `generateComputed:${target.name}`);
      throw e;
    }

    if (key in propData) {
      let e = new Error(`key '${key}' in computed cannot be duplicated with props`);
      (0, _utils.handleError)(e, _this, `generateComputed:${target.name}`);
      throw e;
    }

    if (key in data) {
      let e = new Error(`key '${key}' in computed cannot be duplicated with data()`);
      (0, _utils.handleError)(e, _this, `generateComputed:${target.name}`);
      throw e;
    }

    const v = obj[key];
    if ((0, _utils.isFunction)(v)) return (0, _utils.defComputed)(ret, key, checkUnmount(v));
    (0, _utils.defComputed)(ret, key, checkUnmount(v.get), v.set ? (0, _mobx.action)(key, v.set) : undefined);
  });
  return ret;
}

function bindMethods(ctx, methods) {
  methods && Object.keys(methods).forEach(function (key) {
    // let method = methods[key].bind(ctx);
    // ctx[key] = action(key, method);
    ctx[key] = methods[key].bind(ctx);
  });
}

function bindWatch(ctx, watch) {
  if (!watch) return [];
  return watch.map(function (_ref) {
    let key = _ref.key,
        value = _ref.value;
    return ctx.$watch(key, value);
  });
}

const RESERVED_KEYS = ['state', 'props'];
const LIFECYCLE_HOOKS = ['beforeCreate', 'created', 'beforeMount', 'mounted', 'beforeUpdate', 'updated', 'activated', 'deactivated', 'beforeDestroy', 'destroyed', 'errorCaptured'];

const GLOBAL_TYPES = function () {
  let ret = [_react.default.Component];
  ['EventTarget', 'Event', 'Error', 'Promise', 'RegExp', 'Location', 'IDBFactory', 'History', 'Screen', 'Navigator', 'Storage'].forEach(function (key) {
    return global[key] && ret.push(global[key]);
  });
  return ret;
}();

const RGEX_EVENT = /^on([A-Z]\w+)/;
const RETX_SPECIAL_KEYS = /^[$_]/; // const RGEX_SYNC = /^(\w+)\$sync$/;

const addListener = function (listeners, key, handler) {
  if (!listeners[key]) listeners[key] = [];

  if (!(0, _utils.isFunction)(handler)) {
    if (Array.isArray(handler)) return handler.forEach(function (h) {
      return addListener(listeners, key, h);
    });
    console.warn(`[addListener]${key} is not function!`);
    return;
  }

  listeners[key].push((0, _mobx.action)(key, handler));
};

function initListeners(ctxs, props) {
  let listeners = {};
  ctxs.forEach(function (ctx) {
    LIFECYCLE_HOOKS.forEach(function (key) {
      const name = (0, _utils.camelize)(`hook:${key}`);
      let handler = ctx[key];
      if (!handler && ctx.prototype && ctx.prototype[key]) handler = ctx.prototype[key];
      if (handler) addListener(listeners, name, handler);
    });
  });

  if (props) {
    Object.keys(props).forEach(function (key) {
      const handler = props[key];
      if (!handler) return;

      let _ref2 = key.match(RGEX_EVENT) || [],
          _ref3 = _slicedToArray(_ref2, 2),
          eventName = _ref3[1];

      if (eventName) addListener(listeners, (0, _utils.camelize)(eventName), handler);
    });
  }

  return listeners;
}

function parseProps(target, propTypes) {
  var _this2 = this;

  const propData = {};
  const attrs = {};
  if (!propTypes) propTypes = {};
  Object.getOwnPropertyNames(this.props).forEach(function (key) {
    if (propTypes[key] !== undefined) return (0, _utils.defComputed)(propData, key, function () {
      return _this2.props[key];
    });
    if (['ref'].includes(key) || RETX_SPECIAL_KEYS.test(key)) return;

    if (target.inheritAttrs || target.inheritAttrs === undefined) {
      if (Array.isArray(target.inheritAttrs) && ~target.inheritAttrs.indexOf(key)) return;
      if (~_config.default.inheritAttrs.indexOf(key)) return;
    }

    (0, _utils.defComputed)(attrs, key, function () {
      return _this2.props[key];
    }, null, {
      enumerable: key !== 'children'
    });
  });
  return {
    propData,
    attrs
  };
}

let ReactVueLikeComponent = (0, _mobxReact.observer)(_class = (_temp = _class2 = class ReactVueLikeComponent extends _react.default.Component {
  constructor(_props) {
    super(_props);
    const target = new.target;

    this._createInstance(_props, target);
  }

  _createInstance(props, target, wrapper) {
    var _this3 = this;

    // eslint-disable-next-line
    const propTypes = target.propTypes,
          mixins = target.mixins,
          isRoot = target.isRoot,
          isAbstract = target.isAbstract,
          _target$inherits = target.inherits,
          inherits = _target$inherits === void 0 ? {} : _target$inherits;
    if (wrapper && wrapper.inherits) Object.assign(inherits, wrapper.inherits);

    if (isRoot) {
      this._isVueLikeRoot = true;
    }

    if (isAbstract) this._isVueLikeAbstract = true;
    this._shouldComponentUpdateFn = this.shouldComponentUpdate;
    this.shouldComponentUpdate = this._shouldComponentUpdate;

    const _parseProps$call = parseProps.call(this, target, propTypes),
          propData = _parseProps$call.propData,
          attrs = _parseProps$call.attrs;

    this._isVueLike = true;
    this._ticks = [];
    this._el = null;
    this._mountedPending = [];
    this._isWillMount = false;
    this._isWillUnmount = false;
    this._isActive = true;
    this._isDirty = false;
    this._isRendering = false;
    this.$parent = null;
    this.$context = null;
    this.$root = null;
    this.$children = [];
    this.$attrs = attrs;
    this.$slots = props.$slots || {};
    this.$ref = props.$ref || null;
    this.$options = target;
    if (isRoot && isRoot.isVueLike) this.$vuelike = isRoot;
    if (this.$slots.default === undefined) this.$slots.default = props.children;
    (0, _utils.defComputed)(this, '$el', function () {
      return _this3._el || (_this3._el = _reactDom.default.findDOMNode(_this3));
    }, function (v) {
      throw new Error('ReactVueLike.Component error: $el is readonly!');
    });
    const ctxs = [];

    if (this.$vuelike) {
      ctxs.push(this.$vuelike);
      Array.prototype.push.apply(ctxs, this.$vuelike.plugins.map(function (p) {
        return p.plugin;
      }));
    }

    ctxs.push(target);
    if (wrapper) ctxs.push(wrapper);
    if (mixins) Array.prototype.push.apply(ctxs, mixins);
    let _datas = [];
    let _computed = {};
    let _methods = {};
    let _watch = [];
    let _directives = {};
    let _filters = {};
    let _provideFns = [];
    let _injects = [];
    let _components = {};
    ctxs.forEach(function (ctx) {
      if (!ctx) return;
      if (ctx.components) Object.assign(_components, ctx.components);
      if (ctx.filters) (0, _utils.mergeObject)(_filters, ctx.filters);
      if (ctx.directives) Object.assign(_directives, ctx.directives);
      if (ctx.data) _datas.push(ctx.data);
      if (ctx.computed) Object.assign(_computed, ctx.computed);
      if (ctx.methods) Object.assign(_methods, ctx.methods);
      if (ctx.watch) Object.keys(ctx.watch).forEach(function (key) {
        return _watch.push({
          key,
          value: ctx.watch[key]
        });
      });
      if (ctx.provide) _provideFns.push(ctx.provide);
      if (ctx.inject) ctx.inject.forEach(function (key) {
        return !_injects.includes(key) && _injects.push(key);
      });
    });
    (0, _mobx.extendObservable)(this, propData);
    this.$listeners = initListeners(ctxs, props);
    this.$components = _components;
    this.$filters = _filters;
    this.$directives = _directives;
    this._datas = _datas;
    this._propData = propData;
    this._methods = _methods;
    this._computed = _computed;
    this._watch = _watch;
    this._watched = [];
    this._provideFns = _provideFns;
    this._injects = _injects;

    const _inherits = _objectSpread({}, inherits);

    if (this.$vuelike) {
      _inherits.$vuelike = this.$vuelike;
      Object.assign(_inherits, this.$vuelike.inherits);
    }

    (0, _mobx.extendObservable)(this, {
      _isMounted: false,
      _inherits: _mobx.observable.object(_inherits, {}, {
        deep: false
      }),
      $refs: {}
    }, {}, {
      deep: false
    });
    (0, _utils.defComputed)(this, '$provides', function () {
      if (_this3._provides) return _this3._provides;
      let ret = {};

      _this3._provideFns.forEach(function (p) {
        return Object.assign(ret, (0, _utils.isFunction)(p) ? p.call(_this3) : p);
      });

      return _this3._provides = ret;
    });
    (0, _utils.defComputed)(this, '$isWillUnmount', function () {
      return _this3._isWillUnmount || Boolean(_this3.parent && _this3.parent.$isWillUnmount);
    });
    this.$emit('hook:beforeCreate', props); // if (props.el instanceof Element) this.$mount(props.el);
  }

  _willUnmount() {
    this._isWillUnmount = true;
  }

  _resolvePropRef(ref) {
    const $ref = this.$ref;

    if ($ref) {
      if ((0, _utils.isObject)($ref)) $ref.current = ref;else if ((0, _utils.isFunction)($ref)) $ref(ref);
    }
  }

  _resolveRef(refName, el, key) {
    if (el) el.__vuelike__ = this;

    if (!key) {
      this.$refs[refName] = el;
      return;
    }

    if (typeof key === 'number') {
      if (!this.$refs[refName]) this.$refs[refName] = [];
      this.$refs[refName][key] = el;
      return;
    }

    if (!this.$refs[refName]) this.$refs[refName] = {};
    this.$refs[refName][key] = el;
  }

  _resolveSlot(props, children) {
    let slot = this.$slots[props.name || 'default'];
    let ret;

    if (Array.isArray(slot)) {
      ret = _react.default.Children.map(slot, function (s) {
        return (0, _utils.isFunction)(s) ? s(props) : s;
      });
      if (!ret.length) ret = null;
    } else ret = (0, _utils.isFunction)(slot) ? slot(props) : slot;

    return ret || children || null;
  }

  _eachRenderElement(component, props, children, isRoot) {
    var _this4 = this;

    if (!component) return;

    if (isRoot) {
      this.$emit('hookBeforeRenderRoot', component, props, children);
      if (this.$options.inheritAttrs !== false) this._resolveRootAttrs(component, props, true);

      if (this._isVueLikeAbstract && this.$ref && props.ref === undefined) {
        props.ref = function (ref) {
          return _this4._resolvePropRef(ref);
        };
      }
    } // if (component.__vuelikeAsyncClass && props.vm === undefined) props.vm = this;
    // let scopeId = this.$options.__scopeId;
    // if (scopeId) {
    //   if (!props.className) props.className = scopeId;
    //   else props.className = `${scopeId} ${props.className}`;
    // }

  }

  _resolveRootAttrs(component, props, isInternal) {
    var _this5 = this;

    if (_config.default.useCollect && !isInternal) return props;
    let inheritAttrs = Array.isArray(this.$options.inheritAttrs) ? this.$options.inheritAttrs : _config.default.inheritAttrs;
    const RETX_DOM = /^[a-z][a-z0-9]*$/;
    const isPrimitiveTag = typeof component === 'string' && RETX_DOM.test(component);
    inheritAttrs.forEach(function (key) {
      let v = _this5.props[key];
      if ((0, _utils.isFalsy)(v)) return;

      switch (key) {
        case 'className':
          if (props.className) {
            if (v !== props.className) props.className = `${v} ${props.className}`;
          } else props.className = v;

          break;

        case 'style':
          if (props.style) {
            if (v !== props.style) props.style = Object.assign({}, v, props.style);
          } else props.style = v;

          break;

        default:
          if (props[key] !== undefined) return;
          if (
          /* v === true || */
          isPrimitiveTag && !(0, _utils.isPrimitive)(v)) v = '';
          props[key] = v;
      }

      return props;
    });
  }

  _resolveFilter(filter, filterName) {
    if (!this.$filters) return '';

    try {
      return filter();
    } catch (e) {
      (0, _utils.handleError)(e, this, `filter:${filterName}`);
      return '';
    }
  }

  _resolveWillMount(beforeMount, mounted) {
    var _this6 = this;

    let _pending = (0, _mobx.action)(function () {
      _this6.$root = _this6.$context ? _this6.$context.$root : _this6;

      _this6._resolveStrategies();

      _this6._resolveInherits();

      _this6._resolveEvents();

      _this6._resolveMethods();

      _this6._resolveData();

      _this6._resolveComputed();

      _this6._resolveInject();

      _this6._resolveWatch();

      if (!_this6._isVueLikeAbstract) _this6._resolvePropRef(_this6);

      _this6.$emit('hook:created');

      let pending = _this6._mountedPending;
      _this6._mountedPending = [];
      pending.forEach(function (v) {
        return v();
      });
      beforeMount && beforeMount();
      _this6._isMounted = true;
      mounted && _this6.$nextTick(mounted);
    });

    if (!this.$context || this.$context._isWillMount) _pending();else this.$context._mountedPending.push(_pending);
  }

  _resolveEvents() {
    var _this7 = this;

    if (!this.$vuelike) return;
    let events = {};
    if (this._isVueLikeRoot) Object.assign(events, this.$vuelike.events.app);
    Object.assign(events, this.$vuelike.events.components);
    Object.keys(events).forEach(function (eventName) {
      return addListener(_this7.$listeners, eventName, events[eventName]);
    });
  }

  _resolveStrategies() {
    var _this8 = this;

    let $vuelike = this.$vuelike || this.$context && this.$context.$vuelike || {
      config: {}
    };
    this._optionMergeStrategies = Object.assign({}, _config.default.optionMergeStrategies, $vuelike.config.optionMergeStrategies, this.$options.optionMergeStrategies);

    if (!this._isVueLikeRoot && this.$context) {
      Object.keys(this._optionMergeStrategies).forEach(function (key) {
        let child = _this8[key];
        if (child === undefined) return;

        let ret = _this8._optionMergeStrategies[key](_this8.$context[key], child, _this8, key);

        if (ret !== child) _this8[key] = ret;
      });
    }
  }

  _resolveInherits() {
    var _this9 = this;

    if (!this._isVueLikeRoot && this.$context) {
      if (this.$context._inherits) {
        if (!this._inherits) this._inherits = {};
        Object.assign(this._inherits, this.$context._inherits);
      }
    }

    if (this._inherits) {
      (0, _mobx.action)(function () {
        let res = {};
        Object.keys(_this9._inherits).forEach(function (key) {
          let child = _this9[key];
          let parent = _this9._inherits[key];
          const merge = _this9._optionMergeStrategies[key];
          let v = merge ? merge(parent, child, _this9, key) : parent;

          if (v !== undefined && v !== child) {
            if (child) _this9.$set(_this9, key, v);else (0, _utils.defComputed)(res, key, function () {
              return _this9._inherits[key];
            }, function (v) {
              return _this9._inherits[key] = v;
            });
          }
        });
        (0, _mobx.extendObservable)(_this9, res, {}, {
          deep: false
        });
      })();
    }
  }

  _resolveAction(handler) {
    if (!handler) return handler;
    return (0, _mobx.action)(handler);
  }

  _resolveData() {
    var _this10 = this;

    const _data = {};

    this._datas.forEach(function (data) {
      Object.assign(_data, data.call(_this10, _this10.props));
    });

    this.$data = _data;

    let isPrimitiveObj = function (v) {
      if (v) {
        if (_react.default.isValidElement(v)) return true;
        if (Object.getPrototypeOf(v) !== Object && GLOBAL_TYPES.some(function (t) {
          return v instanceof t;
        })) return true;
      }
    };

    let keys = Object.keys(_data);
    let _shadows = this.$options.__shadows;

    if (!_shadows) {
      this.$options.__shadows = _shadows = keys.filter(function (key) {
        if (RESERVED_KEYS.includes(key)) {
          let e = new Error(`key '${key}' in data() is reserved keywords`);
          (0, _utils.handleError)(e, _this10, `_resolveData:${_this10.$options.name}`);
          throw e;
        }

        if (key in _this10._propData) {
          let e = new Error(`key '${key}' in data() cannot be duplicated with props`);
          (0, _utils.handleError)(e, _this10, `_resolveData:${_this10.$options.name}`);
          throw e;
        }

        return RETX_SPECIAL_KEYS.test(key) || isPrimitiveObj(_data[key]);
      });
    }

    let deeps = {};
    let shadows = {};
    keys.forEach(function (key) {
      if (~_shadows.indexOf(key)) shadows[key] = _data[key];else deeps[key] = _data[key];
    });
    (0, _mobx.extendObservable)(this, deeps, {}, {
      deep: true
    });
    (0, _mobx.extendObservable)(this, shadows, {}, {
      deep: false
    });
  }

  _resolveComputed() {
    let _computed = generateComputed(this._computed, this._propData, this.$data, this._methods, this.$options);

    (0, _mobx.extendObservable)(this, _computed);
  }

  _resolveWatch() {
    this._watched = bindWatch(this, this._watch);
  }

  _resolveMethods() {
    var _this11 = this;

    bindMethods(this, this._methods);
    const pMethods = {};
    Object.getOwnPropertyNames(this.$options.prototype).filter(function (key) {
      return !ReactVueLikeComponent.prototype[key];
    }).map(function (key) {
      return (0, _utils.isFunction)(_this11[key]) && (pMethods[key] = _this11[key]);
    });
    bindMethods(this, pMethods);
  }

  _resolveParent() {
    var _this12 = this;

    if (this._isVueLikeRoot) return;
    if (this.props.$parent) this.$parent = this.props.$parent;else (0, _utils.iterativeParent)(this, function (vm) {
      return !vm._isVueLikeAbstract && (_this12.$parent = vm);
    }, function (vm) {
      return vm._isVueLike;
    });
    this.$context = this.props.$context || this.$parent;
    if (this.$parent) this.$parent.$children.push(this);
  }

  _resolveInject() {
    var _this13 = this;

    if (!this.$context) return;

    try {
      const injects = this._injects.slice();

      if (injects.length) {
        (0, _utils.iterativeParent)(this.$context, function (vm) {
          return Object.keys(vm.$provides).some(function (key) {
            let idx = injects.indexOf(key);

            if (~idx) {
              let v = vm.$provides[key];
              if (v !== undefined) _this13.$set(_this13, key, v);
              injects.splice(idx, 1);
            }

            return !injects.length;
          });
        }, function (vm) {
          return vm._isVueLike;
        }, true); // if (process.env.NODE_ENV !== 'production') {
        //   injects.forEach(key => warn(`inject '${key}' not found it's provide!`, this));
        // }
      }
    } catch (e) {
      (0, _utils.handleError)(e, this, 'resolveInject');
      throw e;
    }
  }

  _resolveComp(compName) {
    let comp;
    if (this.$components) comp = this.$components[compName];

    if (!this._isVueLikeRoot && !comp && this.$root.$components) {
      comp = this.$root.$components[compName];
    }

    if (!_utils.isProduction && !comp) (0, _utils.warn)(`can not resolve component '${compName}'!`, this);
    return comp || compName;
  }

  _resolveUpdated() {
    this._el = null;
  }

  _resolveDestroy() {
    var _this14 = this;

    // this._flushTicks();
    this._watched.forEach(function (v) {
      return v && v();
    });

    const $ref = this.props.$ref;

    if (!this._isVueLikeAbstract && $ref) {
      if ((0, _utils.isObject)($ref)) $ref.current = null;else if ((0, _utils.isFunction)($ref)) $ref(null);
    }

    if (this.$parent) {
      const idx = this.$parent.$children.findIndex(function (c) {
        return c === _this14;
      });
      if (~idx) this.$parent.$children.splice(idx, 1);
    }

    const el = this.$el;
    if (el && el.__vuelike__) delete el.__vuelike__;
  }

  _flushTicks() {
    if (!this._ticks.length) return;

    const ticks = this._ticks.slice();

    this._ticks = [];
    setTimeout(function () {
      return ticks.forEach(function (v) {
        return v();
      });
    }, 0);
  }

  _callProto(key, args) {
    let fn = this.$options.vuelikeProto[key];
    return fn && fn.apply(this, args);
  }

  _callListener(eventName, handlers, args) {
    const map = {
      hookMounted: 'componentDidMount',
      hookBeforeUpdate: 'getSnapshotBeforeUpdate',
      hookUpdated: 'componentDidUpdate',
      hookActivated: 'componentDidActivate',
      hookDeactivated: 'componentWillUnactivate',
      hookBeforeDestroy: 'componentWillUnmount',
      hookErrorCaptured: 'componentDidCatch'
    };

    try {
      if (!handlers) return;
      if ((0, _utils.isFunction)(handlers)) return handlers.call.apply(handlers, [this].concat(_toConsumableArray(args)));
      let ret;

      for (let handler of handlers) {
        ret = handler.call.apply(handler, [this].concat(_toConsumableArray(args), [ret]));
      }

      let protoEventName = map[eventName];

      if (protoEventName && this.$options.vuelikeProto[protoEventName]) {
        ret = this.$options.vuelikeProto[protoEventName].apply(this, args);
      }

      return ret;
    } catch (e) {
      (0, _utils.handleError)(e, this, `$emit:${eventName}`);
      throw e;
    }
  }

  _checkActive(callback, args) {
    if (this._isActive && !this.$isWillUnmount) {
      this._isDirty = false;
      return callback.apply(this, args);
    }

    this._isDirty = true;
  }

  _shouldComponentUpdate(nextProps, nextState) {
    let ret1 = this._checkActive(this._shouldComponentUpdateFn, [nextProps, nextState]);

    let shouldComponentUpdate1 = this.$options.vuelikeProto.shouldComponentUpdate;
    let ret2 = !shouldComponentUpdate1 || shouldComponentUpdate1(nextProps, nextState);
    return Boolean(ret1 || ret2);
  }

  static data(props) {
    return {};
  }

  static provide() {
    return {};
  }

  // $mount(elementOrSelector) {
  //   if (!elementOrSelector) throw new Error('$mount error: elementOrSelector can not be null!');
  //   let el;
  //   let sc = false;
  //   if (typeof elementOrSelector === 'string') el = document.getElementById(elementOrSelector);
  //   else if (elementOrSelector instanceof Element) el = elementOrSelector;
  //   else {
  //     el = document.createElement('div');
  //     sc = true;
  //   }
  //   // throw new Error(`$mount error: elementOrSelector ${elementOrSelector} is not support type!`);
  //   let instance = this;
  //   const ReactVueLikeProxy = function ReactVueLikeProxy(props) {
  //     return instance;
  //   };
  //   ReactVueLikeProxy.prototype = React.Component.prototype;
  //   ReactVueLikeProxy.dispalyName = this.$options.dispalyName || this.$options.name;
  //   ReactDOM.render(React.createElement(ReactVueLikeProxy, this.props), el);
  //   return sc ? el : instance;
  // }
  $destroy() {
    _reactDom.default.unmountComponentAtNode(this.$el);
  }

  $runAction() {
    return _mobx.runInAction.apply(void 0, arguments);
  }

  $nextTick(cb, ctx) {
    var _this15 = this,
        _this$_ticks;

    if (!cb) return new Promise(function (resolve) {
      return _this15._ticks.push((0, _mobx.action)(resolve));
    });
    if (!Array.isArray(cb)) cb = [cb];
    cb.forEach(function (c, i) {
      return cb[i] = (0, _mobx.action)(ctx ? c.bind(ctx) : c);
    });

    (_this$_ticks = this._ticks).push.apply(_this$_ticks, _toConsumableArray(cb));

    setTimeout(function () {
      return _this15._flushTicks();
    }, 0);
  }

  $watch(expOrFn, callback) {
    let options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    if (!expOrFn || !callback) return;
    callback = (0, _mobx.action)(checkUnmount(callback).bind(this));
    if (options.deep && options.deep !== undefined) console.warn('$watch not support deep option!');

    if (typeof expOrFn === 'string') {
      let _parseExpr = (0, _utils.parseExpr)(this, expOrFn),
          obj = _parseExpr.obj,
          key = _parseExpr.key;

      if (obj && key) {
        return (0, _mobx.observe)(obj, key, function (change) {
          return callback(change.newValue, change.oldValue);
        }, options.immediate);
      }
    } else if ((0, _utils.isFunction)(expOrFn)) {
      let oldValue;
      return (0, _mobx.when)(function () {
        let newValue = expOrFn();
        let ret = oldValue !== newValue;
        oldValue = newValue;
        return ret;
      }, callback);
    }
  }

  $computed(target, expr, value) {
    let force = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

    let _parseExpr2 = (0, _utils.parseExpr)(target, expr),
        obj = _parseExpr2.obj,
        key = _parseExpr2.key;

    if (obj && key) {
      if (!force && obj[key]) {
        let e = new Error(`key '${expr}' has aleady exist!`);
        (0, _utils.handleError)(e, this, `$computed:${expr}`);
        throw e;
      }

      let computedObj = {};
      if ((0, _utils.isFunction)(value)) (0, _utils.defComputed)(computedObj, key, checkUnmount(value));else (0, _utils.defComputed)(computedObj, key, checkUnmount(value.get), value.set);
      (0, _mobx.extendObservable)(obj, computedObj);
    }
  }

  $set(target, expr, value) {
    if (!(0, _mobx.isObservable)(target)) {
      if ((0, _utils.isPlainObject)(expr)) return Object.assign(target, expr);

      let _parseExpr3 = (0, _utils.parseExpr)(target, expr),
          obj = _parseExpr3.obj,
          key = _parseExpr3.key;

      if (obj && key) obj[key] = value;
      return;
    }

    if ((0, _utils.isPlainObject)(expr)) {
      const _expr = {};
      Object.keys(expr).forEach(function (key) {
        value = expr[key];
        if (value && !(0, _mobx.isObservable)(value) && ((0, _utils.isPlainObject)(value) || Array.isArray(value))) value = (0, _mobx.observable)(value);
        _expr[key] = value;
      });
      return (0, _mobx.runInAction)(function () {
        return (0, _mobx.set)(target, _expr);
      });
    }

    let _parseExpr4 = (0, _utils.parseExpr)(target, expr),
        obj = _parseExpr4.obj,
        key = _parseExpr4.key;

    if (obj && key) {
      if (value && !(0, _mobx.isObservable)(value) && ((0, _utils.isPlainObject)(value) || Array.isArray(value))) value = (0, _mobx.observable)(value);
      (0, _mobx.runInAction)(function () {
        return (0, _mobx.set)(obj, key, value);
      });
    }
  }

  $delete(target, expr) {
    if ((0, _utils.isObject)(expr)) {
      return Object.keys(expr).forEach(function (key) {
        return (0, _mobx.remove)(target, expr[key]);
      });
    }

    let _parseExpr5 = (0, _utils.parseExpr)(target, expr),
        obj = _parseExpr5.obj,
        key = _parseExpr5.key;

    if (obj && key) (0, _mobx.remove)(obj, key);
  }

  $emit(eventName) {
    eventName = (0, _utils.camelize)(eventName);

    for (var _len = arguments.length, payload = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      payload[_key - 1] = arguments[_key];
    }

    return this._callListener(eventName, this.$listeners[eventName], payload);
  }

  $on(eventName, handler) {
    var _this16 = this;

    eventName = (0, _utils.camelize)(eventName);
    if (!this.$listeners[eventName]) this.$listeners[eventName] = [];
    this.$listeners[eventName].push(handler);
    return function () {
      return _this16.$off(eventName, handler);
    };
  }

  $off(eventName, handler) {
    eventName = (0, _utils.camelize)(eventName);

    if (handler) {
      const handlers = this.$listeners[eventName];

      if (handlers) {
        const idx = handlers.findIndex(function (v) {
          return v === handler || v === handler._source;
        });
        if (~idx) handlers.splice(idx, 1);
      }

      if (handlers.length) return;
    }

    delete this.$listeners[eventName];
  }

  $once(eventName, handler) {
    const off = this.$on(eventName, function () {
      off();

      for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      return handler.call.apply(handler, [this].concat(args));
    });
  }

  render() {
    if (!this._isMounted) return null;
    let node;
    this._isRendering = true;

    try {
      let renderFn = this.$options.vuelikeProto.render;
      node = renderFn && renderFn.call(this) || null;
    } catch (ex) {
      (0, _utils.handleError)(ex, this, 'render');
      node = this.renderError && this.renderError(ex);
    }

    this._isRendering = false;
    return node;
  }

  renderError(ex) {
    let node = null;

    try {
      let _renderErrorFn = this.$options.vuelikeProto.renderError;
      node = _renderErrorFn && _renderErrorFn.call(this, ex) || null;
    } catch (ex) {
      (0, _utils.handleError)(ex, this, 'renderError');
    }

    return node;
  }

  getSnapshotBeforeUpdate(prevProps, prevState) {
    this.$emit('hook:beforeUpdate');
    return null;
  }

  componentDidMount() {
    var _this17 = this;

    this._resolveParent();

    this._isWillMount = true;

    this._resolveWillMount(function () {
      return _this17.$emit('hook:beforeMount');
    }, function () {
      return _this17.$emit('hook:mounted');
    });
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    this._resolveUpdated();

    const ret = this.$emit('hook:updated');

    this._flushTicks();

    return ret;
  }

  componentDidActivate() {
    if (this._isActive) return;
    this._isActive = true;
    const ret = this.$emit('hook:activated');
    if (this._isDirty) this.forceUpdate();
    return ret;
  }

  componentWillUnactivate() {
    if (!this._isActive) return;
    this._isActive = false;
    return this.$emit('hook:deactivated');
  }

  componentWillUnmount() {
    const ret = this.$emit('hook:beforeDestroy');

    this._resolveDestroy();

    return ret;
  }

  componentDidCatch(error, info) {
    return this.$emit('hook:errorCaptured', error, this, info);
  }

}, _defineProperty(_class2, "isRoot", false), _defineProperty(_class2, "isAbstract", false), _defineProperty(_class2, "inheritAttrs", true), _defineProperty(_class2, "inheritMergeStrategies", {}), _defineProperty(_class2, "props", {}), _defineProperty(_class2, "components", {}), _defineProperty(_class2, "directives", {}), _defineProperty(_class2, "filters", {}), _defineProperty(_class2, "inject", []), _defineProperty(_class2, "computed", {}), _defineProperty(_class2, "watch", {}), _defineProperty(_class2, "methods", {}), _temp)) || _class;

const ALIAS_METHODS = {
  _resolveComp: '_c',
  _resolveAction: '_a',
  _resolveRef: '_r',
  _resolveFilter: '_f',
  _resolveRootAttrs: '_o',
  _resolveSlot: '_s'
};
Object.keys(ALIAS_METHODS).forEach(function (key) {
  return (0, _utils.innumerable)(ReactVueLikeComponent.prototype, ALIAS_METHODS[key], ReactVueLikeComponent.prototype[key]);
});
(0, _utils.innumerable)(ReactVueLikeComponent, _utils.VUE_LIKE_CLASS, true);
var _default = ReactVueLikeComponent;
exports.default = _default;