"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/web.dom.iterable");

var _mobx = require("mobx");

var _config = _interopRequireDefault(require("./config"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function normalizeComponentNames(comps, prefix) {
  let parentKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  const COMP_REGX = /^[A-Z][A-Za-z]+/;
  let ret = {};
  Object.keys(comps).forEach(function (key) {
    if (!COMP_REGX.test(key)) return;
    const comp = comps[key];

    if (!comp) {
      console.warn(`component [${key}] is not exist!`);
      return;
    }

    ret[`${prefix}${parentKey}${key}`] = comp;
    Object.assign(ret, normalizeComponentNames(comp, prefix, `${parentKey}${key}`));
  });
  return ret;
}

class PluginArray extends Array {
  get(name) {
    let p = this.find(function (p) {
      return p.name === name;
    });
    return p ? p.plugin : null;
  }

}

class ReactVueLike {
  constructor() {
    let options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    this.isVueLike = true;
    this.options = options;
    this.plugins = new PluginArray();
    this.mixins = [];
    this.components = {};
    this.filters = {};
    this.directives = {};
    this.hocs = [];
    this.events = {
      components: {},
      app: {}
    };
    this.inherits = {};
    this.config = _objectSpread({}, _config.default);
  }

  configure() {
    let options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    (0, _utils.mergeObject)(this.config, options);

    if (this.config.enforceActions !== undefined) {
      (0, _mobx.configure)({
        enforceActions: _config.default.enforceActions ? 'observed' : 'never'
      });
    }
  }

  use(plugin) {
    var _this = this;

    let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    if (!plugin) return; // if (!plugin.name) throw new Error('[ReactVueLike.use]plugin \'name\' can not be empty!');

    let install = (0, _utils.isFunction)(plugin) ? plugin : plugin.install ? plugin.install.bind(plugin) : null;
    if (!install) throw Error('[ReactVueLike.use]plugin need has \'install\' method!');
    let ret = install(this, options);

    let afterInstall = function () {
      _this.plugins.push({
        name: plugin.name,
        plugin,
        options
      });

      return function () {
        let ret;

        let idx = _this.plugins.find(function (p) {
          return p.plugin === plugin;
        });

        if (~idx) ret = _this.plugins.splice(idx, 1);
        return ret;
      };
    };

    return ret instanceof Promise ? ret.then(function () {
      return afterInstall();
    }) : afterInstall();
  }

  mixin(mixin) {
    if (!mixin) return;
    this.mixins.push(mixin);
    return function () {
      let ret;
      let idx = this.mixins.find(function (v) {
        return v === mixin;
      });
      if (~idx) ret = this.plugins.splice(idx, 1);
      return ret;
    };
  }

  component(keyOrComponents) {
    let componentOrPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

    if ((0, _utils.isPlainObject)(keyOrComponents)) {
      Object.assign(this.components, normalizeComponentNames(keyOrComponents, componentOrPrefix));
      return keyOrComponents;
    }

    return this.components[keyOrComponents] = componentOrPrefix;
  }

  filter(keyOrFilters, filter) {
    if ((0, _utils.isPlainObject)(keyOrFilters)) {
      (0, _utils.mergeObject)(this.filters, keyOrFilters);
      return keyOrFilters;
    }

    return this.filters[keyOrFilters] = filter;
  }

  directive(keyOrDirectives, directive) {
    if ((0, _utils.isPlainObject)(keyOrDirectives)) {
      Object.assign(this.directives, keyOrDirectives);
      return keyOrDirectives;
    }

    return this.directives[keyOrDirectives] = directive;
  }

  _off(name, listener, events) {
    if (!name) throw new Error('[ReactVueLike]on name can not be empty!');
    let listeners = events[name];
    if (!listeners) return;
    let old;

    if (!listener) {
      old = events[name];
      events[name] = [];
      return old;
    }

    const idx = listeners.findIndex(function (v) {
      return v === listener;
    });
    if (~idx) old = listeners.splice(idx, 1);
    return old;
  }

  on(name, listener, isComponent) {
    var _this2 = this;

    if (!name) throw new Error('[ReactVueLike]on name can not be empty!');
    if (!listener) throw new Error('[ReactVueLike]on listener can not be null!');
    let events = isComponent ? this.events.components : this.events.app;
    let listeners = events[name];
    if (!listeners) events[name] = listeners = [];
    const v = listeners.find(function (v) {
      return v === listener;
    });
    if (!v) listeners.push(listener);
    return function () {
      return _this2._off(name, listener, events);
    };
  }

  off(name, listener, isComponent) {
    let events = isComponent ? this.events.components : this.events.app;
    return this._off(name, listener, events);
  }

  inherit(keyOrInherits, value) {
    if (!keyOrInherits) return;

    if ((0, _utils.isPlainObject)(keyOrInherits)) {
      Object.assign(this.inherits, keyOrInherits);
      return keyOrInherits;
    }

    return this.inherits[keyOrInherits] = value;
  }

}

var _default = ReactVueLike;
exports.default = _default;